import { Notice } from "obsidian";
import { Command } from "./Command";
import type { Goodreads } from "@data-sources/Goodreads";
import type { Book } from "@src/types";

const GOODREADS_URL_PATTERN = /https:\/\/www.goodreads.com\/book\/show\/(\d+)/;

export class CreateBookFromGoodreadsUrlCommand extends Command {
	constructor(
		private readonly goodreads: Goodreads,
		private readonly createEntry: (book: Book) => void | PromiseLike<void>
	) {
		super(
			"create-book-from-goodreads-url",
			"Create Book from Goodreads URL"
		);
	}

	async callback() {
		const url = await navigator.clipboard.readText();
		const legacyId = parseInt(
			GOODREADS_URL_PATTERN.exec(url)?.[1] ?? "",
			10
		);

		if (isNaN(legacyId)) {
			new Notice("Clipboard does not contain a valid Goodreads URL.");
			return;
		}

		let book: Book;
		try {
			// eslint-disable-next-line prefer-const
			book = await this.goodreads.getBookByLegacyId(legacyId);
		} catch (error) {
			console.error("Failed to get book from Goodreads:", error);
			new Notice(
				"Failed to get book from Goodreads. Check console for details."
			);
			return;
		}

		try {
			await this.createEntry(book);
		} catch (error) {
			console.error("Failed to create book:", error);
			new Notice("Failed to create book. Check console for details.");
			return;
		}

		new Notice("Book created from Goodreads URL.");
	}
}
